

var pt = (function($) {

	var table = $('#apt-table'),
		labels = $('#apt-labels'),
		labelBody = labels.find('.apt-body'),
		labelRow = labelBody.find('.apt-feature:first').clone(true),
		column = $('#apt-table').find('.apt-column:first'),
		columnBody = column.find('.apt-body'),
		columnRow = columnBody.find('.apt-feature:first').clone(true),
		counter = $('#apt-table').find('.apt-column').length;

	if (counter > 1) {
		$('#apt-table .apt-column').find('.apt-icon-remove').show();
	} 

	// column remover trigger
	table.click(function(e) {
		var that = $(e.target);

		if (that.is('.apt-icon-remove')) {
			pt.removeColumn(that.closest('.apt-column'));
		}
	});

	// row remover trigger
	labels.click(function(e) {
		var that = $(e.target);

		if (that.is('.apt-icon-remove')) {
			pt.removeRow(that.closest('.apt-feature'));
		}
	});

	function _resetColumnSortable()
	{
		table.sortable('destroy');

		if ($('#apt-table').find('.apt-column').length > 1) {
			table.sortable({
				cancel: ".apt-icon, input, textarea"
			});
		}
	}


	function _resetRowSortable()
	{
		var features = columnBody.find('.apt-feature'),
			idx, ptBody, ptClones = [];
		
		labelBody.sortable('destroy');

		if (features.length > 1) {
			labelBody.sortable({
				handle: ".apt-icon-move",
				items: ".apt-feature",
				helper: 'clone',
				opacity: 0.6,

				start: function(e, ui) {
					idx = labelBody.find('.apt-feature').index(ui.item);
					ptBody = table.find('.apt-body');

					ptBody.each(function() {
						var feature = $('.apt-feature', this).eq(idx),
							clone = feature.clone(true);

						feature.css('visibility', 'hidden');
						clone.addClass('apt-feature-clone').css({ 
							position: 'absolute',
							opacity: 0.6
						}).appendTo(this);
						ptClones.push(clone);
					});
				},

				sort: function(e, ui) {
					$.each(ptClones, function(i, clone) {
						clone.css({
							top: ui.position.top
						});
					});
				},

				stop: function(e, ui) {
					var newIdx = labelBody.find('.apt-feature').index(ui.item);

					$.each(ptClones, function(i, clone) {
						clone.remove();
					});
					ptClones = [];

					
					ptBody.each(function() {
						var features = $('.apt-feature', this),
							featureToMove = $('.apt-feature', this).eq(idx);

						if (idx !== newIdx) {
							if (idx > newIdx) {
								featureToMove.insertBefore($('.apt-feature', this).eq(newIdx));
							} else {
								featureToMove.insertAfter($('.apt-feature', this).eq(newIdx));
							}
						}

						featureToMove.css('visibility', 'visible');
					});
				}
			});
		}
	}

	function _resetColumn(col)
	{
		col.find('[name]').each(function() {
			var that = $(this);
			if (that.is(':checkbox')) {
				that.removeAttr('checked');
			} else {
				that.val('');
			}
		});
	}

	_resetColumnSortable();
	_resetRowSortable();

	return {

		preview: function() {
			$.post(ajaxurl, {
					action: 'pt_get_preview',
					post_id: $('#post_ID').val()
				}, function(r) {
					if (r.OK) {
						if ($('#apt-preview').length) {
							$('#apt-preview').replaceWith(r.info);
						} else {
							$(r.info).insertAfter('#apt-price-table');
						}
					}
				}, 'json'
			);
		},

		addRow: function() {
			var html = columnRow.clone(true).find('textarea').text('').end().wrap('<div>').parent().html();

			labelBody.append(labelRow.clone(true).find('input').val('').end());
			table.find('.apt-body').each(function() {
				var id = $(this).closest('.apt-column').attr('id').replace(/apt-column-/, '');
				$(this).append(html.replace(/pt\[col\]\[\d+\]/ig, 'pt[col][' + id + ']'));
			});
			_resetRowSortable();
		},

		removeRow: function(row) {
			var idx;

			if (!isNaN(parseInt(row, 10))) { 
				idx = row;
				row = labelBody.find('.apt-feature').eq(idx);
			} else {
				idx = labelBody.find('.apt-feature').index(row);
			}
			
			row.remove();
			table.find('.apt-body').each(function() {
				$('.apt-feature', this).eq(idx).remove();
			});
			_resetRowSortable();
		},

		addColumn: function() {
			var html = $('#apt-table').find('.apt-column:first').clone(true).wrap('<div>').parent().html();
			
			col = $(html.replace(/pt\[col\]\[\d+\]/ig, 'pt[col][' + (++counter) + ']'));

			// every column is going to have unique identifier, order matters, not value
			col.attr('id', 'apt-column-' + counter);
			table.append(col);

			_resetColumn(col);

			// turn removal ability back
			var cols = $('#apt-table').find('.apt-column');
			if (cols.length === 2) {
				cols.find('.apt-icon-remove').show();
			} 

			_resetColumnSortable();
		},

		removeColumn: function(col) {
			var idx;

			if (!isNaN(parseInt(col, 10))) {
				idx = col;
				col = $('#apt-table').find('.apt-column').eq(idx);
			}

			col.remove();
			
			// disable ability to remove column if there's only one left
			var cols = $('#apt-table').find('.apt-column');
			if (cols.length === 1) {
				cols.find('.apt-icon-remove').hide();
			} 

			_resetColumnSortable();
		}

	};

}(jQuery));