<?php /*
Plugin Name: EZ Price Tables
Plugin URI: http://www.ezpricetables.com
Description: Easily create nice, professional Price Tables with a single shortcode 
Version: 1.1a
Author: yjtung
Author URI: http://www.ezpricetables.com
License: GPL2
*/

include(dirname(__FILE__) . "/i8/class.Plugino.php");

class PT extends Plugino {

	public $addons = array("Pro");
	
	function a__after_setup_theme()
	{
		register_post_type('price_table',
			array(
				'labels' => array(
					'name' 					=> __( 'Price Tables', 'pt' ),
					'singular_name' 		=> __( 'Price Table', 'pt' ),
					'add_new' 				=> __( 'New Price Table', 'pt' ),
					'add_new_item'			=> __( 'New Price Table', 'pt' ),
					'edit' 					=> __( 'Edit', 'pt' ),
					'edit_item'				=> __( 'Edit Price Table', 'pt' ),
					'new_item' 				=> __( 'New Price Table', 'pt' ),
					'view' 					=> __( 'View Price Table', 'pt' ),
					'view_item' 			=> __( 'View Price Table', 'pt' ),
					'search_items'		 	=> __( 'Search Price Tables', 'pt' ),
					'not_found' 			=> __( 'No Price Tables found', 'pt' ),
					'not_found_in_trash' 	=> __( 'No Price Tables found in Trash', 'pt' ),
					'parent' 				=> __( 'Parent Price Table', 'pt' ),
				),
				'public' => false,
				'show_ui' => true,
				'menu_icon' => "$this->url/images/table_icon.png",
				//'menu_position' => 5,
				'publicly_queryable' => false,
				'exclude_from_search' => true,
				'hierarchical' => false,
				'query_var' => false,
				'supports' => array('title'), 
				'register_meta_box_cb' => array($this, '_add_price_table_metaboxes'),
			)
		);
	}


	function f__manage_posts_columns($cols)
	{	
		global $typenow;
		if ($typenow != 'price_table')
			return $cols;
		
		$cols = array(
			'cb' => $cols['cb'],
			'title' => $cols['title'],
			'template' => "Template",
			'shortcode' => "Shortcode",
			'date' => $cols['date'],
		);

		return $cols;
	}
	
	
	function a_10_2__manage_posts_custom_column($col_name, $post_id)
	{
		global $typenow;
		if ($typenow != 'price_table')
			return $cols;
			
		if ($col_name == 'shortcode') {
			?><code><strong>[price_table id="<?php echo $post_id; ?>"]</strong></code><?php 
		} elseif ($col_name == 'template') {
			echo "Default";
		}
	}


	function f_10_2__post_row_actions($actions, $post)
	{
		if ($post->post_type != 'price_table') {
			return $actions;	
		}

		unset($actions['inline hide-if-no-js']);
		return $actions;
	}


	function _add_price_table_metaboxes($post)
	{
		$table = get_post_meta($post->ID, '_tableinfo', true);
		
		add_meta_box('apt-price-table', __("Price Table"), array($this, '_price_table_metabox'), 'price_table', 'normal', 'core');

		if (!empty($table)) {
			add_meta_box('apt-preview', __("Preview"), array($this, '_preview_metabox'), 'price_table', 'normal', 'core');
		}
		
		add_meta_box('apt-description', __("Tips"), array($this, '_description_metabox'), 'price_table', 'side', 'core');
		add_meta_box('apt-attributes', __("Attributes"), array($this, '_attributes_metabox'), 'price_table', 'side', 'core');
	}


	function _preview_metabox($post)
	{
		echo $this->_generate($post->ID);
	}


	function _price_table_metabox($post)
	{
		$table = get_post_meta($post->ID, '_tableinfo', true);

		$label = $col = array();

		if (!empty($table)) {
			$table = stripslashes_deep($table);
			extract($table);
		}

		include("$this->path/tpl/price_table_metabox.php");
	}


	function _description_metabox($post)
	{
		include("$this->path/tpl/description_metabox.php");
	}


	function _attributes_metabox($post)
	{
		include("$this->path/tpl/attributes_metabox.php");
	}


	function a__admin_print_styles()
	{
		if (self::is_addedit_page('price_table')) {
			wp_enqueue_style('style', "$this->url/css/style.css");
		}
	}


	function a__admin_print_scripts()
	{
		if (self::is_addedit_page('price_table')) {
			wp_deregister_script('autosave'); // we do not need autosave on price table pages
			wp_enqueue_script('jquery-ui-sortable');
			wp_enqueue_script('script', "$this->url/js/script.js", array('jquery'), $this->info['Version'], true);
		}
	}


	function a_10_2__wp_insert_post($post_id, $post)
	{
		if ($post->post_type == 'price_table' || isset($_POST['pt'])) {
			update_post_meta($post_id, '_tableinfo', $_POST['pt']);
		}
	}

	
	function a__wp_ajax_pt_get_preview()
	{
		$post_id = intval($_POST['post_id']);
		$table = get_post_meta($post_id, '_tableinfo', true);

		if (empty($table)) {
			$this->json(array('OK' => 0));
		}

		ob_start();

		$output = ob_get_clean();
	}


	function sc__price_table($attr)
	{
		return $this->_generate($attr['id']);
	}


	function the($key, $var, $default = false)
	{
		if ($var && isset($var[$key])) {
			echo esc_attr($var[$key]);
		} else {
			echo $default;
		}
	}


	function _generate($post_id)
	{
		ob_start();
		$table = get_post_meta($post_id, '_tableinfo', true);
		if (!empty($table)) {
			$table = stripslashes_deep($table);

			$url = $this->url;
			$path = $this->path;
			$template_path = "$this->path/tpl/preview_metabox.php";
			
			if (isset($table['template']) && $table['template'] != 'default') {
				extract(apply_filters('pt_template_path', compact('path', 'url', 'template_path'), $table['template']), EXTR_IF_EXISTS);
			}

			include($template_path);
		}
		return ob_get_clean();
	}


	function _minify_css($source)
	{
		require_once("$this->path/tools/cssmin.php");	
		return CssMin::minify($source, array
        (
			"remove-empty-blocks"           => true,
			"remove-empty-rulesets"         => true,
			"remove-last-semicolons"        => true,
			"convert-css3-properties"       => true,
			"convert-font-weight-values"    => true, // new in v2.0.2
			"convert-named-color-values"    => true, // new in v2.0.2
			"convert-hsl-color-values"      => true, // new in v2.0.2
			"convert-rgb-color-values"      => true, // new in v2.0.2; was "convert-color-values" in v2.0.1
			"compress-color-values"         => true,
			"compress-unit-values"          => true,
			"emulate-css3-variables"        => true
        ));	
	}


}

new PT;





















